@@ -16,8 +16,9 @@ struct State {
     rebase: bool,
     pull: bool,
     push: bool,
+    fixup: git_stack::config::Fixup,
     dry_run: bool,
-    backup_capacity: Option<usize>,
+    snapshot_capacity: Option<usize>,
 
     show_format: git_stack::config::Format,
     show_stacked: bool,
@@ -38,13 +39,28 @@ impl State {
             log::trace!("`--pull` implies `--rebase`");
             rebase = true;
         }
+        let rebase = rebase;
+
+        let fixup = if args.fixup.is_some() || rebase {
+            repo_config.fixup()
+        } else {
+            // Assume the user is only wanting to show the tree and not modify it.
+            let no_op = git_stack::config::Fixup::Ignore;
+            if no_op != repo_config.fixup() {
+                log::trace!(
+                    "Ignoring `fixup={}` without an explicit `--rebase` or `--fixup`",
+                    repo_config.fixup()
+                );
+            }
+            no_op
+        };
         let push = args.push;
         let protected = git_stack::git::ProtectedBranches::new(
             repo_config.protected_branches().iter().map(|s| s.as_str()),
         )
         .with_code(proc_exit::Code::CONFIG_ERR)?;
         let dry_run = args.dry_run;
-        let backup_capacity = repo_config.capacity();
+        let snapshot_capacity = repo_config.capacity();
 
         let show_format = repo_config.show_format();
         let show_stacked = repo_config.show_stacked();
@@ -155,8 +171,9 @@ impl State {
             rebase,
             pull,
             push,
+            fixup,
             dry_run,
-            backup_capacity,
+            snapshot_capacity,
 
             show_format,
             show_stacked,
@@ -212,9 +229,14 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
     let repo = git2::Repository::discover(&cwd).with_code(proc_exit::Code::USAGE_ERR)?;
     let repo = git_stack::git::GitRepo::new(repo);
     let mut state = State::new(repo, args)?;
+    let mut stash_id = None;
 
     if state.pull {
+        if stash_id.is_none() {
+            stash_id = git_stack::git::stash_push(&mut state.repo, "branch-stash");
+        }
         if state.repo.is_dirty() {
+            git_stack::git::stash_pop(&mut state.repo, stash_id);
             return Err(proc_exit::Code::USAGE_ERR.with_message("Working tree is dirty, aborting"));
         }
 
@@ -272,34 +294,45 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
         }
     }
 
-    const BACKUP_NAME: &str = "git-stack";
+    const STASH_STACK_NAME: &str = "git-stack";
     let mut success = true;
     let mut backed_up = false;
     if state.rebase {
+        if stash_id.is_none() {
+            stash_id = git_stack::git::stash_push(&mut state.repo, "branch-stash");
+        }
         if state.repo.is_dirty() {
+            git_stack::git::stash_pop(&mut state.repo, stash_id);
             return Err(proc_exit::Code::USAGE_ERR.with_message("Working tree is dirty, aborting"));
         }
 
-        let mut backups = git_stack::backup::Stack::new(BACKUP_NAME, &state.repo);
-        backups.capacity(state.backup_capacity);
-        let mut backup = git_stack::backup::Backup::from_repo(&state.repo)
+        let mut snapshots = git_stack::stash::Stack::new(STASH_STACK_NAME, &state.repo);
+        snapshots.capacity(state.snapshot_capacity);
+        let mut snapshot = git_stack::stash::Snapshot::from_repo(&state.repo)
             .with_code(proc_exit::Code::FAILURE)?;
-        backup.insert_parent(&state.repo, &state.branches, &state.protected_branches);
+        snapshot.insert_parent(&state.repo, &state.branches, &state.protected_branches);
         if !state.dry_run {
-            backups.push(backup)?;
+            snapshots.push(snapshot)?;
             backed_up = true;
         }
 
-        let head_branch = state
+        let mut head_branch = state
             .repo
             .head_branch()
             .ok_or_else(|| eyre::eyre!("Must not be in a detached HEAD state."))
-            .with_code(proc_exit::Code::USAGE_ERR)?;
+            .with_code(proc_exit::Code::USAGE_ERR)?
+            .name;
 
         let scripts: Result<Vec<_>, proc_exit::Exit> = state
             .stacks
             .iter()
-            .map(|stack| plan_rebase(&state, stack).with_code(proc_exit::Code::FAILURE))
+            .map(|stack| {
+                let script = plan_rebase(&state, stack).with_code(proc_exit::Code::FAILURE)?;
+                if script.is_branch_deleted(&head_branch) {
+                    head_branch = stack.onto.name.clone();
+                }
+                Ok(script)
+            })
             .collect();
         let scripts = scripts?;
 
@@ -315,7 +348,7 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
             }
         }
         executor
-            .close(&mut state.repo, &head_branch.name)
+            .close(&mut state.repo, &head_branch)
             .with_code(proc_exit::Code::FAILURE)?;
         state.update().with_code(proc_exit::Code::FAILURE)?;
     }
@@ -327,8 +360,10 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
 
     show(&state, colored_stdout).with_code(proc_exit::Code::FAILURE)?;
 
+    git_stack::git::stash_pop(&mut state.repo, stash_id);
+
     if backed_up {
-        log::info!("To undo, run `git branch-backup pop {}`", BACKUP_NAME);
+        log::info!("To undo, run `git branch-stash pop {}`", STASH_STACK_NAME);
     }
 
     if !success {
@@ -339,12 +374,18 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
 }
 
 fn plan_rebase(state: &State, stack: &StackState) -> eyre::Result<git_stack::git::Script> {
-    let mut graphed_branches = stack.graphed_branches();
-    let mut root = git_stack::graph::Node::new(state.head_commit.clone(), &mut graphed_branches);
-    root = root.extend_branches(&state.repo, graphed_branches)?;
+    let graphed_branches = stack.graphed_branches();
+    let base_commit = state
+        .repo
+        .find_commit(stack.base.id)
+        .expect("base branch is valid");
+    let root = git_stack::graph::Node::from_branches(&state.repo, graphed_branches)?;
+    let mut root = root.extend(&state.repo, git_stack::graph::Node::new(base_commit))?;
     git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches);
 
     git_stack::graph::rebase_branches(&mut root, stack.onto.id);
+    git_stack::graph::drop_by_tree_id(&mut root);
+    git_stack::graph::fixup(&mut root, state.fixup);
 
     let script = git_stack::graph::to_script(&root);
 
@@ -357,8 +398,11 @@ fn push(state: &mut State) -> eyre::Result<()> {
         let stack_graphed_branches = stack.graphed_branches();
         graphed_branches.extend(stack_graphed_branches.into_iter().flat_map(|(_, b)| b));
     }
-    let mut root = git_stack::graph::Node::new(state.head_commit.clone(), &mut graphed_branches);
-    root = root.extend_branches(&state.repo, graphed_branches)?;
+    let root = git_stack::graph::Node::from_branches(&state.repo, graphed_branches)?;
+    let mut root = root.extend(
+        &state.repo,
+        git_stack::graph::Node::new(state.head_commit.clone()),
+    )?;
 
     git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches);
     git_stack::graph::pushable(&mut root);
@@ -373,22 +417,26 @@ fn show(state: &State, colored_stdout: bool) -> eyre::Result<()> {
         .stacks
         .iter()
         .map(|stack| -> eyre::Result<git_stack::graph::Node> {
-            let mut graphed_branches = stack.graphed_branches();
-            let mut root =
-                git_stack::graph::Node::new(state.head_commit.clone(), &mut graphed_branches);
-            root = root.extend_branches(&state.repo, graphed_branches)?;
+            let graphed_branches = stack.graphed_branches();
+            let base_commit = state
+                .repo
+                .find_commit(stack.base.id)
+                .expect("base branch is valid");
+            let root = git_stack::graph::Node::from_branches(&state.repo, graphed_branches)?;
+            let mut root = root.extend(&state.repo, git_stack::graph::Node::new(base_commit))?;
             git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches);
 
             if state.dry_run {
                 // Show as-if we performed all mutations
                 git_stack::graph::rebase_branches(&mut root, stack.onto.id);
+                git_stack::graph::drop_by_tree_id(&mut root);
+                git_stack::graph::fixup(&mut root, state.fixup);
             }
 
             eyre::Result::Ok(root)
         });
     let mut root = roots.next().unwrap_or_else(|| {
-        let mut graphed_branches = git_stack::git::Branches::new(None.into_iter());
-        let root = git_stack::graph::Node::new(state.head_commit.clone(), &mut graphed_branches);
+        let root = git_stack::graph::Node::new(state.head_commit.clone());
         Ok(root)
     })?;
     for other in roots {
@@ -680,7 +728,7 @@ fn drop_branches(
             if branch.name == potential_head {
                 continue;
             } else if head_branch_name == Some(branch.name.as_str()) {
-                // Dom't leave HEAD detached but instead switch to the branch we pulled
+                // Don't leave HEAD detached but instead switch to the branch we pulled
                 log::trace!("git switch {}", potential_head);
                 if !dry_run {
                     repo.switch(potential_head)?;
@@ -860,7 +908,7 @@ fn to_tree<'r>(
 ) -> Tree<'r> {
     let mut weight = if node.action.is_protected() {
         Weight::Protected(0)
-    } else if node.local_commit.id == head_branch.id {
+    } else if node.commit.id == head_branch.id {
         Weight::Head(0)
     } else {
         Weight::Commit(0)
@@ -1015,11 +1063,11 @@ impl<'r> std::fmt::Display for RenderNode<'r> {
                 let abbrev_id = self
                     .repo
                     .raw()
-                    .find_object(node.local_commit.id, None)
+                    .find_object(node.commit.id, None)
                     .unwrap()
                     .short_id()
                     .unwrap();
-                let style = if self.head_branch.id == node.local_commit.id {
+                let style = if self.head_branch.id == node.commit.id {
                     self.palette.highlight
                 } else if node.action.is_protected() {
                     self.palette.info
@@ -1065,13 +1113,13 @@ impl<'r> std::fmt::Display for RenderNode<'r> {
                 format_commit_status(self.repo, node, self.palette)
             )?;
 
-            let summary = String::from_utf8_lossy(&node.local_commit.summary);
+            let summary = String::from_utf8_lossy(&node.commit.summary);
             if node.action.is_protected() {
                 write!(f, "{}", self.palette.hint.paint(summary))?;
-            } else if node.local_commit.fixup_summary().is_some() {
+            } else if node.commit.fixup_summary().is_some() {
                 // Needs to be squashed
                 write!(f, "{}", self.palette.warn.paint(summary))?;
-            } else if node.local_commit.wip_summary().is_some() {
+            } else if node.commit.wip_summary().is_some() {
                 // Not for pushing implicitly
                 write!(f, "{}", self.palette.error.paint(summary))?;
             } else {
@@ -1138,7 +1186,7 @@ fn format_branch_status<'d>(
         format!("")
     } else if 1 < repo
         .raw()
-        .find_commit(node.local_commit.id)
+        .find_commit(node.commit.id)
         .unwrap()
         .parent_count()
     {
@@ -1183,18 +1231,18 @@ fn format_commit_status<'d>(
     node: &'d git_stack::graph::Node,
     palette: &'d Palette,
 ) -> String {
-    // See format_commit_status
+    // See format_branch_status
     if node.action.is_protected() {
         format!("")
     } else if node.action.is_delete() {
-        format!("{} ", palette.warn.paint("(drop)"))
+        format!(" {}", palette.error.paint("(drop)"))
     } else if 1 < repo
         .raw()
-        .find_commit(node.local_commit.id)
+        .find_commit(node.commit.id)
         .unwrap()
         .parent_count()
     {
-        format!("{} ", palette.error.paint("(merge commit)"))
+        format!(" {}", palette.error.paint("(merge commit)"))
     } else {
         format!("")
     }
