@@ -9,6 +9,7 @@ pub struct RepoConfig {
     pub pull_remote: Option<String>,
     pub show_format: Option<Format>,
     pub show_stacked: Option<bool>,
+    pub fixup: Option<Fixup>,
 
     pub capacity: Option<usize>,
 }
@@ -19,7 +20,8 @@ static PUSH_REMOTE_FIELD: &str = "stack.push-remote";
 static PULL_REMOTE_FIELD: &str = "stack.pull-remote";
 static FORMAT_FIELD: &str = "stack.show-format";
 static STACKED_FIELD: &str = "stack.show-stacked";
-static BACKUP_CAPACITY_FIELD: &str = "branch-backup.capacity";
+static FIXUP_FIELD: &str = "stack.fixup";
+static BACKUP_CAPACITY_FIELD: &str = "branch-stash.capacity";
 
 static DEFAULT_PROTECTED_BRANCHES: [&str; 4] = ["main", "master", "dev", "stable"];
 const DEFAULT_CAPACITY: usize = 30;
@@ -127,6 +129,10 @@ impl RepoConfig {
                 }
             } else if key == STACKED_FIELD {
                 config.show_stacked = Some(value.as_ref().map(|v| v == "true").unwrap_or(true));
+            } else if key == FIXUP_FIELD {
+                if let Some(value) = value.as_ref().and_then(|v| FromStr::from_str(v).ok()) {
+                    config.fixup = Some(value);
+                }
             } else if key == BACKUP_CAPACITY_FIELD {
                 config.capacity = value.as_deref().and_then(|s| s.parse::<usize>().ok());
             } else {
@@ -208,6 +214,12 @@ impl RepoConfig {
             .and_then(|s| FromStr::from_str(s).ok());
 
         let show_stacked = config.get_bool(STACKED_FIELD).ok();
+
+        let fixup = config
+            .get_str(FIXUP_FIELD)
+            .ok()
+            .and_then(|s| FromStr::from_str(s).ok());
+
         let capacity = config
             .get_i64(BACKUP_CAPACITY_FIELD)
             .map(|i| i as usize)
@@ -220,6 +232,7 @@ impl RepoConfig {
             stack,
             show_format,
             show_stacked,
+            fixup,
 
             capacity,
         }
@@ -257,6 +270,7 @@ impl RepoConfig {
         self.stack = other.stack.or(self.stack);
         self.show_format = other.show_format.or(self.show_format);
         self.show_stacked = other.show_stacked.or(self.show_stacked);
+        self.fixup = other.fixup.or(self.fixup);
         self.capacity = other.capacity.or(self.capacity);
 
         self
@@ -288,6 +302,10 @@ impl RepoConfig {
         self.show_stacked.unwrap_or(true)
     }
 
+    pub fn fixup(&self) -> Fixup {
+        self.fixup.unwrap_or_else(Default::default)
+    }
+
     pub fn capacity(&self) -> Option<usize> {
         let capacity = self.capacity.unwrap_or(DEFAULT_CAPACITY);
         (capacity != 0).then(|| capacity)
@@ -335,6 +353,12 @@ impl std::fmt::Display for RepoConfig {
             STACKED_FIELD.split_once(".").unwrap().1,
             self.show_stacked()
         )?;
+        writeln!(
+            f,
+            "\t{}={}",
+            FIXUP_FIELD.split_once(".").unwrap().1,
+            self.fixup()
+        )?;
         writeln!(f, "[{}]", BACKUP_CAPACITY_FIELD.split_once(".").unwrap().0)?;
         writeln!(
             f,
@@ -351,18 +375,48 @@ fn git_dir_config(repo: &git2::Repository) -> std::path::PathBuf {
 }
 
 fn default_branch(config: &git2::Config) -> &str {
-    config.get_str("init.defaultStack").ok().unwrap_or("main")
+    config.get_str("init.defaultBranch").ok().unwrap_or("main")
 }
 
-arg_enum! {
-    #[derive(Debug, Copy, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
-    #[serde(rename_all = "kebab-case")]
-    pub enum Format {
-        Silent,
-        Branches,
-        BranchCommits,
-        Commits,
-        Debug,
+#[derive(Debug, Copy, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum Format {
+    Silent,
+    Branches,
+    BranchCommits,
+    Commits,
+    Debug,
+}
+
+impl Format {
+    pub fn variants() -> [&'static str; 5] {
+        ["silent", "branches", "branch-commits", "commits", "debug"]
+    }
+}
+
+impl std::str::FromStr for Format {
+    type Err = String;
+    fn from_str(s: &str) -> ::std::result::Result<Self, Self::Err> {
+        match s {
+            "silent" => Ok(Format::Silent),
+            "branches" => Ok(Format::Branches),
+            "branch-commits" => Ok(Format::BranchCommits),
+            "commits" => Ok(Format::Commits),
+            "debug" => Ok(Format::Debug),
+            _ => Err(format!("valid values: {}", Self::variants().join(", "))),
+        }
+    }
+}
+
+impl std::fmt::Display for Format {
+    fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
+        match self {
+            Format::Silent => "silent".fmt(f),
+            Format::Branches => "branches".fmt(f),
+            Format::BranchCommits => "branch-commits".fmt(f),
+            Format::Commits => "commits".fmt(f),
+            Format::Debug => "debug".fmt(f),
+        }
     }
 }
 
@@ -372,14 +426,42 @@ impl Default for Format {
     }
 }
 
-arg_enum! {
-    #[derive(Debug, Copy, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
-    #[serde(rename_all = "kebab-case")]
-    pub enum Stack {
-        Current,
-        Dependents,
-        Descendants,
-        All,
+#[derive(Debug, Copy, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum Stack {
+    Current,
+    Dependents,
+    Descendants,
+    All,
+}
+
+impl Stack {
+    pub fn variants() -> [&'static str; 4] {
+        ["current", "dependents", "descendants", "all"]
+    }
+}
+
+impl std::str::FromStr for Stack {
+    type Err = String;
+    fn from_str(s: &str) -> ::std::result::Result<Self, Self::Err> {
+        match s {
+            "current" => Ok(Stack::Current),
+            "dependents" => Ok(Stack::Dependents),
+            "descendants" => Ok(Stack::Descendants),
+            "all" => Ok(Stack::All),
+            _ => Err(format!("valid values: {}", Self::variants().join(", "))),
+        }
+    }
+}
+
+impl std::fmt::Display for Stack {
+    fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
+        match self {
+            Stack::Current => "current".fmt(f),
+            Stack::Dependents => "dependents".fmt(f),
+            Stack::Descendants => "descendants".fmt(f),
+            Stack::All => "all".fmt(f),
+        }
     }
 }
 
@@ -388,3 +470,45 @@ impl Default for Stack {
         Stack::All
     }
 }
+
+#[derive(Debug, Copy, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum Fixup {
+    Ignore,
+    Move,
+    Squash,
+}
+
+impl Fixup {
+    pub fn variants() -> [&'static str; 3] {
+        ["ignore", "move", "squash"]
+    }
+}
+
+impl std::str::FromStr for Fixup {
+    type Err = String;
+    fn from_str(s: &str) -> ::std::result::Result<Self, Self::Err> {
+        match s {
+            "ignore" => Ok(Fixup::Ignore),
+            "move" => Ok(Fixup::Move),
+            "squash" => Ok(Fixup::Squash),
+            _ => Err(format!("valid values: {}", Self::variants().join(", "))),
+        }
+    }
+}
+
+impl std::fmt::Display for Fixup {
+    fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
+        match self {
+            Fixup::Ignore => "ignore".fmt(f),
+            Fixup::Move => "move".fmt(f),
+            Fixup::Squash => "squash".fmt(f),
+        }
+    }
+}
+
+impl Default for Fixup {
+    fn default() -> Self {
+        Fixup::Move
+    }
+}
