@@ -1,6 +1,7 @@
 #[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub enum Action {
     Pick,
+    Squash,
     Protected,
     Delete,
 }
@@ -10,6 +11,10 @@ impl Action {
         matches!(self, Action::Pick)
     }
 
+    pub fn is_squash(&self) -> bool {
+        matches!(self, Action::Squash)
+    }
+
     pub fn is_protected(&self) -> bool {
         matches!(self, Action::Protected)
     }
